// C++ Fossil implementation
// V1.1 by G.Iachello 1992 - Public Domain
// Pubblicato nel numero 28 di Telematicus

#include <stdio.h>
#include <dos.h>
//#include <conio.h>
#include "fossil.hpp"

FossilDriver::FossilDriver (unsigned int iport)
{
	port=iport;

	intvect=getvect(FOSSILE_INT);
	maxfunc=((unsigned char far*)(intvect))[8];

	if (!Initialize()) printf("Error initializing Fossil driver.\n");
	if (maxfunc>=0x1b) GetDriverInformation();
	if (maxfunc>=0x7) GetSystemTimerParams();
	ctrlCflag=FALSE;
}


FossilDriver::~FossilDriver(void)
{
	Deinitialize();
}

StatusBits FossilDriver::SetBaudRate(BAUDRATE br,PARITY pa, STOPBITS sb, CHARLENGTH cl)
{
	CheckCall(0x0);
	unsigned regAL;
	unsigned tAX;
	regAL=cl | sb << 2 | pa << 3 | br << 5;
	_DX=port;
	_AL=regAL;
	_AH=0;
	geninterrupt(FOSSILE_INT);
	tAX=_AX;
	statbits=*((StatusBits*)&tAX);
	return statbits;
}

StatusBits FossilDriver::TxCharWait(unsigned char c)
{
	CheckCall(0x1);
	unsigned tAX;
	_DX=port;
	_AL=c;
	_AH=1;
	geninterrupt(FOSSILE_INT);
	tAX=_AX;
	statbits=*((StatusBits*)&tAX);
	return statbits;
}

BOOL FossilDriver::TxCharNoWait(unsigned char c)
{
	CheckCall(0x0b);
	_DX=port;
	_AL=c;
	_AH=0x0b;
	geninterrupt(FOSSILE_INT);
	if (_AX) return TRUE;
	return FALSE;
}

unsigned char FossilDriver::RxCharWait(void)
{
	CheckCall(0x2);
	_DX=port;
	_AH=2;
	geninterrupt(FOSSILE_INT);
	last=_AL;
	return last;
}

unsigned char FossilDriver::PeekInChar(void)
{
	CheckCall(0xc);
	_DX=port;
	_AH=0x0c;
	geninterrupt(FOSSILE_INT);
	return _AL;
}

BOOL FossilDriver::IsInChar(void)
{
	CheckCall(0xc);
	_DX=port;
	_AH=0x0c;
	geninterrupt(FOSSILE_INT);
	if (_AX==0xffff) return FALSE;
	return TRUE;
}


StatusBits FossilDriver::StatusRequest(void)
{
	CheckCall(0x3);
	unsigned tAX;
	_DX=port;
	_AH=3;
	geninterrupt(FOSSILE_INT);
	tAX=_AX;
	statbits=*((StatusBits*)&tAX);
	return statbits;
}

BOOL FossilDriver::Initialize(void)
{
	CheckCall(0x4);
	unsigned char t1,t2;
	_DX=port;
	_AH=4;
	geninterrupt(FOSSILE_INT);
	if (_AX==0x1954) {
		t1=_BL;
		t2=_BH;
		maxfunc=t1;
		revision=t2;
		status=TRUE;
	} else status=FALSE;
	return status;
}

void FossilDriver::Deinitialize(void)
{
	CheckCall(0x5);
	_DX=port;
	_AH=5;
	geninterrupt(FOSSILE_INT);
}

void FossilDriver::GetDriverInformation(void)
{
	CheckCall(0x1b);
	driverinfo.size=sizeof(DriverInfo);
	REGPACK regs;
	regs.r_es=FP_SEG((void far *)&driverinfo);
	regs.r_di=FP_OFF((void far *)&driverinfo);
	regs.r_dx=port;
	regs.r_cx=sizeof(DriverInfo);
	regs.r_ax=0x1b00;
	intr(FOSSILE_INT,&regs);
}

void FossilDriver::RaiseDTR(void)
{
	CheckCall(0x6);
	_DX=port;
	_AH=6;
	_AL=1;
	geninterrupt(FOSSILE_INT);
}

void FossilDriver::LowerDTR(void)
{
	CheckCall(0x6);
	_DX=port;
	_AH=6;
	_AL=0;
	geninterrupt(FOSSILE_INT);
}

void FossilDriver::FlushOutputBuffer(void)
{
	CheckCall(0x8);
	_DX=port;
	_AH=8;
	geninterrupt(FOSSILE_INT);
}

void FossilDriver::PurgeOutputBuffer(void)
{
	CheckCall(0x9);
	_DX=port;
	_AH=9;
	geninterrupt(FOSSILE_INT);
}

void FossilDriver::PurgeInputBuffer(void)
{
	CheckCall(0xa);
	_DX=port;
	_AH=0x0a;
	geninterrupt(FOSSILE_INT);
}

unsigned int FossilDriver::ReadBlock(char far* buffer,unsigned int count)
{
	CheckCall(0x18);
	REGPACK regs;
	regs.r_es=FP_SEG((void far *)buffer);
	regs.r_di=FP_OFF((void far *)buffer);
	regs.r_dx=port;
	regs.r_cx=count;
	regs.r_ax=0x1800;
	intr(FOSSILE_INT,&regs);
	return regs.r_ax;
}

unsigned int FossilDriver::WriteBlock(char far* buffer,unsigned int count)
{
	CheckCall(0x19);
	REGPACK regs;
	regs.r_es=FP_SEG((void far *)buffer);
	regs.r_di=FP_OFF((void far *)buffer);
	regs.r_dx=port;
	regs.r_cx=count;
	regs.r_ax=0x1900;
	intr(FOSSILE_INT,&regs);
	return regs.r_ax;
}

void FossilDriver::StartBreak(void)
{
	CheckCall(0x1a);
	_DX=port;
	_AH=0x1a;
	_AL=1;
	geninterrupt(FOSSILE_INT);
}

void FossilDriver::StopBreak(void)
{
	CheckCall(0x1a);
	_DX=port;
	_AH=0x1a;
	_AL=0;
	geninterrupt(FOSSILE_INT);
}

BOOL FossilDriver::SetTimerFunc(void (far * func)(),BOOL status)
{
	CheckCall(0x16);
	REGPACK regs;
	regs.r_es=FP_SEG((void far *)func);
	regs.r_dx=FP_OFF((void far *)func);
	if (status) regs.r_ax=0x1601; else regs.r_ax=0x1600;

	intr(FOSSILE_INT,&regs);
	if (regs.r_ax) return FALSE; else return TRUE;
}

void FossilDriver::GetSystemTimerParams(void)
{
	CheckCall(0x7);
	_DX=port;
	_AH=7;
	geninterrupt(FOSSILE_INT);
	timerIntNo=_AL;
	ticksxsec=_AH;
	msxtick=_DX;
}

void FossilDriver::SetFlowControl(unsigned char m)
{
	CheckCall(0x0f);
	_DX=port;
	_AL=m | 0xf0;
	_AH=0x0f;
	geninterrupt(FOSSILE_INT);
}

BOOL FossilDriver::SetCtrlCK(BOOL status)
{
	CheckCall(0x10);
	if (status) _AL=1; else _AL=0;
	_DX=port;
	_AH=0x10;
	geninterrupt(FOSSILE_INT);
	if (_AX | ctrlCflag) { ctrlCflag=FALSE; return TRUE; }
	return FALSE;
}

BOOL FossilDriver::SetTransmitter(BOOL status)
{
	CheckCall(0x10);
	if (status) _AL=2; else _AL=0;
	_DX=port;
	_AH=0x10;
	geninterrupt(FOSSILE_INT);
	if (_AX) { ctrlCflag=TRUE; return TRUE; }
	return FALSE;
}

void FossilDriver::SetWatchdog(BOOL status)
{
	CheckCall(0x14);
	if (status) _AL=1; else _AL=0;
	_DX=port;
	_AH=0x14;
	geninterrupt(FOSSILE_INT);
}

unsigned char FossilDriver::ReadKeybWait(void)
{
	CheckCall(0x0e);
	_AH=0x0e;
	geninterrupt(FOSSILE_INT);
	return _AX;
}

unsigned char FossilDriver::PeekKeybChar(void)
{
	CheckCall(0x0d);
	_AH=0x0d;
	geninterrupt(FOSSILE_INT);
	return _AX;
}

BOOL FossilDriver::IsKeybChar(void)
{
	CheckCall(0x0d);
	_AH=0x0d;
	geninterrupt(FOSSILE_INT);
	if (_AX==0xffff) return FALSE;
	return TRUE;
}

void FossilDriver::SetCursor(unsigned int row, unsigned int col)
{
	CheckCall(0x11);
	_AH=0x11;
	_DL=col;
	_DH=row;
	geninterrupt(FOSSILE_INT);
}

int FossilDriver::GetCursorRow(void)
{
	CheckCall(0x12);
	_AH=0x12;
	geninterrupt(FOSSILE_INT);
	return _DH;
}

int FossilDriver::GetCursorCol(void)
{
	CheckCall(0x12);
	_AH=0x12;
	geninterrupt(FOSSILE_INT);
	return _DL;
}

void FossilDriver::WriteAnsiChar(unsigned char c)
{
	CheckCall(0x13);
	_AH=0x13;
	_AL=c;
	geninterrupt(FOSSILE_INT);
}

void FossilDriver::WriteChar(unsigned char c)
{
	CheckCall(0x15);
	_AH=0x15;
	_AL=c;
	geninterrupt(FOSSILE_INT);
}

void FossilDriver::WarmBoot(void)
{
	Reboot(1);
}

void FossilDriver::ColdBoot(void)
{
	Reboot(0);
}

void FossilDriver::Reboot(unsigned int type)
{
	CheckCall(0x17);
	_AH=0x17;
	_AL=type;
	geninterrupt(FOSSILE_INT);
}
